"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var az_api_maps_loader_1 = require("../../services/az-api-maps-loader");
var az_api_maps_wrapper_1 = require("../../services/az-api-maps-wrapper");
var Rx_1 = require("rxjs/Rx");
var AZMapChildComponent_1 = require("../../entities/AZMapChildComponent");
var DEFAULT_ZOOM = 4;
var MAP_ELEMENT_ID = 'az-map-container';
var AZMapComponent = (function (_super) {
    __extends(AZMapComponent, _super);
    function AZMapComponent(_element, _renderer, _wrapper, _loader) {
        _super.call(this);
        this._element = _element;
        this._renderer = _renderer;
        this._wrapper = _wrapper;
        this._loader = _loader;
    }
    AZMapComponent.prototype.optionsAvailable = function () {
        return ['backgroundColor', 'center', 'disableDefaultUI', 'disableDoubleClickZoom', 'draggable', 'draggableCursor', 'draggingCursor', 'heading', 'keyboardShortcuts',
            'mapMaker', 'mapTypeControl', 'mapTypeControlOptions', 'mapTypeId', 'maxZoom', 'minZoom', 'noClear', 'overviewMapControl', 'overviewMapControlOptions',
            'panControl', 'panControlOptions', 'rotateControl', 'rotateControlOptions', 'scaleControl', 'scaleControlOptions', 'scrollwheel', 'streetView',
            'streetViewControl', 'streetViewControlOptions', 'styles', 'tilt', 'zoom', 'zoomControl', 'zoomControlOptions'];
    };
    AZMapComponent.prototype.ngOnInit = function () {
        var _this = this;
        this._loader.load().then(function () {
            var mapElement = _this._renderer.createElement(_this._element.nativeElement, 'div');
            mapElement.id = MAP_ELEMENT_ID;
            _this.mapId = _this._wrapper.loadMap(mapElement, _this.getMapOptions());
            _this._element.nativeElement.setAttribute('map-id', _this.mapId);
            _this.buildDirection();
        });
    };
    AZMapComponent.prototype.ngOnChanges = function (changes) {
        var _this = this;
        this._loader.load().then(function () {
            if (_this._lastBuiltDirection) {
                _this.buildDirection();
            }
        });
    };
    AZMapComponent.prototype.buildDirection = function () {
        var _this = this;
        if (!this.direction || this._lastBuiltDirection === this.direction) {
            return;
        }
        this._lastBuiltDirection = this.direction;
        this.direction.subscribe(function (coordinates) {
            if (coordinates.length < 2) {
                return;
            }
            var request = _this._createDirectionRequest(coordinates);
            var directionsService = new google.maps.DirectionsService();
            var directionDisplay = new google.maps.DirectionsRenderer();
            _this._removeLastDirection();
            _this._lastDirectionDisplay = directionDisplay;
            directionDisplay.setMap(_this._wrapper.getNativeMap(_this.mapId));
            directionsService.route(request, function (result, status) {
                if (status === google.maps.DirectionsStatus.OK) {
                    directionDisplay.setDirections(result);
                }
            });
        });
    };
    AZMapComponent.prototype.getMapOptions = function () {
        this._checkMapOptionsValidity();
        return this.getComponentOptions();
    };
    AZMapComponent.prototype._checkMapOptionsValidity = function () {
        if (!this.centerLat) {
            throw Error('The center longitude of the map must be specified, using az-map-center-lat property on az-map component.');
        }
        if (!this.centerLng) {
            throw Error('The center latitude of the map must be specified, using az-map-center-lng property on az-map component.');
        }
        this.zoom = this.zoom || DEFAULT_ZOOM;
        this.center = {
            lat: this.centerLat,
            lng: this.centerLng
        };
    };
    AZMapComponent.prototype._createDirectionRequest = function (coordinates) {
        var request = {
            origin: coordinates.shift(),
            destination: coordinates.pop(),
            travelMode: google.maps.TravelMode.DRIVING,
            optimizeWaypoints: true
        };
        if (coordinates.length) {
            request.waypoints = this._generateWaypointsFromCoordinates(coordinates);
        }
        return request;
    };
    AZMapComponent.prototype._generateWaypointsFromCoordinates = function (coordinates) {
        return coordinates.map(function (coord) {
            return {
                location: new google.maps.LatLng(coord.lat, coord.lng),
                stopover: false
            };
        });
    };
    AZMapComponent.prototype._removeLastDirection = function () {
        if (this._lastDirectionDisplay) {
            this._lastDirectionDisplay.setMap(null);
        }
    };
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "centerLat", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "centerLng", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Rx_1.Observable)
    ], AZMapComponent.prototype, "direction", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], AZMapComponent.prototype, "backgroundColor", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "disableDefaultUI", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "disableDoubleClickZoom", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "draggable", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], AZMapComponent.prototype, "draggableCursor", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', String)
    ], AZMapComponent.prototype, "draggingCursor", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "heading", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "keyboardShortcuts", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "mapMaker", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "mapTypeControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "mapTypeControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "mapTypeId", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "maxZoom", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "minZoom", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "noClear", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "overviewMapControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "overviewMapControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "panControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "panControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "rotateControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "rotateControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "scaleControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "scaleControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "scrollwheel", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "streetView", void 0);
    __decorate([
        //StreetViewPanorama, cannot specified, because it is a class, not an interface, and it tries to find google namespace when it is not loaded.
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "streetViewControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "streetViewControlOptions", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Array)
    ], AZMapComponent.prototype, "styles", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "tilt", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Number)
    ], AZMapComponent.prototype, "zoom", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Boolean)
    ], AZMapComponent.prototype, "zoomControl", void 0);
    __decorate([
        core_1.Input(), 
        __metadata('design:type', Object)
    ], AZMapComponent.prototype, "zoomControlOptions", void 0);
    __decorate([
        core_1.ViewChild('div'), 
        __metadata('design:type', HTMLBodyElement)
    ], AZMapComponent.prototype, "container", void 0);
    AZMapComponent = __decorate([
        core_1.Component({
            selector: 'az-map',
            template: "<div id=\"#div\">\n    <ng-content></ng-content>\n    </div>"
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef, core_1.Renderer, az_api_maps_wrapper_1.AZApiMapsWrapperService, az_api_maps_loader_1.AZApiMapsLoaderService])
    ], AZMapComponent);
    return AZMapComponent;
}(AZMapChildComponent_1.AZMapChildComponent));
exports.AZMapComponent = AZMapComponent;
//# sourceMappingURL=az-map.component.js.map